@@ -32,7 +32,7 @@
 #include <vector>
 #include "v4l2-compliance.h"
 
-static int checkQCtrl(struct node *node, struct test_queryctrl &qctrl)
+static int checkQCtrl(struct node *node, struct test_query_ext_ctrl &qctrl)
 {
 	struct v4l2_querymenu qmenu;
 	__u32 fl = qctrl.flags;
@@ -41,7 +41,7 @@ static int checkQCtrl(struct node *node, struct test_queryctrl &qctrl)
 	int i;
 
 	qctrl.menu_mask = 0;
-	if (check_ustring(qctrl.name, sizeof(qctrl.name)))
+	if (check_string(qctrl.name, sizeof(qctrl.name)))
 		return fail("invalid name\n");
 	info("checking v4l2_queryctrl of control '%s' (0x%08x)\n", qctrl.name, qctrl.id);
 	if (qctrl.id & V4L2_CTRL_FLAG_NEXT_CTRL)
@@ -63,11 +63,12 @@ static int checkQCtrl(struct node *node, struct test_queryctrl &qctrl)
 	case V4L2_CTRL_TYPE_MENU:
 	case V4L2_CTRL_TYPE_INTEGER_MENU:
 		if (qctrl.step != 1)
-			return fail("invalid step value %d\n", qctrl.step);
+			return fail("invalid step value %lld\n", qctrl.step);
 		if (qctrl.minimum < 0)
 			return fail("min < 0\n");
 		/* fall through */
 	case V4L2_CTRL_TYPE_INTEGER:
+	case V4L2_CTRL_TYPE_INTEGER64:
 		if (qctrl.default_value < qctrl.minimum ||
 		    qctrl.default_value > qctrl.maximum)
 			return fail("def < min || def > max\n");
@@ -79,7 +80,8 @@ static int checkQCtrl(struct node *node, struct test_queryctrl &qctrl)
 			return fail("step == 0\n");
 		if (qctrl.step < 0)
 			return fail("step < 0\n");
-		if ((unsigned)qctrl.step > (unsigned)(qctrl.maximum - qctrl.minimum))
+		if ((unsigned)qctrl.step > (unsigned)(qctrl.maximum - qctrl.minimum) &&
+		    qctrl.maximum != qctrl.minimum)
 			return fail("step > max - min\n");
 		if ((qctrl.maximum - qctrl.minimum) % qctrl.step) {
 			// This really should be a fail, but there are so few
@@ -99,7 +101,6 @@ static int checkQCtrl(struct node *node, struct test_queryctrl &qctrl)
 			return fail("default_value is out of range for a bitmask control\n");
 		break;
 	case V4L2_CTRL_TYPE_CTRL_CLASS:
-	case V4L2_CTRL_TYPE_INTEGER64:
 	case V4L2_CTRL_TYPE_BUTTON:
 		if (qctrl.minimum || qctrl.maximum || qctrl.step || qctrl.default_value)
 			return fail("non-zero min/max/step/def\n");
@@ -141,12 +142,11 @@ static int checkQCtrl(struct node *node, struct test_queryctrl &qctrl)
 		qmenu.id = qctrl.id;
 		qmenu.index = qctrl.minimum;
 		ret = doioctl(node, VIDIOC_QUERYMENU, &qmenu);
-		if (ret != EINVAL)
+		if (ret != EINVAL && ret != ENOTTY)
 			return fail("can do querymenu on a non-menu control\n");
 		return 0;
 	}
-	if (qctrl.maximum >= 32)
-		return fail("currently more than 32 menu items are not supported\n");
+	bool have_default_value = false;
 	for (i = 0; i <= qctrl.maximum + 1; i++) {
 		memset(&qmenu, 0xff, sizeof(qmenu));
 		qmenu.id = qctrl.id;
@@ -165,21 +165,24 @@ static int checkQCtrl(struct node *node, struct test_queryctrl &qctrl)
 			return fail("invalid menu name\n");
 		if (qmenu.reserved)
 			return fail("reserved is non-zero\n");
-		qctrl.menu_mask |= 1 << i;
+		if (i == qctrl.default_value)
+			have_default_value = true;
+		if (i < 64)
+			qctrl.menu_mask |= 1ULL << i;
 	}
 	if (qctrl.menu_mask == 0)
 		return fail("no menu items found\n");
-	if (!(qctrl.menu_mask & (1 << qctrl.default_value)))
-		return fail("the default_value is an invalid menu item\n");
+	fail_on_test(!have_default_value);
 	return 0;
 }
 
-int testQueryControls(struct node *node)
+int testQueryExtControls(struct node *node)
 {
-	struct test_queryctrl qctrl;
+	struct test_query_ext_ctrl qctrl;
 	__u32 id = 0;
+	int result = 0;
 	int ret;
-	__u32 ctrl_class = 0;
+	__u32 which = 0;
 	bool found_ctrl_class = false;
 	unsigned user_controls = 0;
 	unsigned priv_user_controls = 0;
@@ -190,11 +193,11 @@ int testQueryControls(struct node *node)
 	for (;;) {
 		memset(&qctrl, 0xff, sizeof(qctrl));
 		qctrl.id = id | V4L2_CTRL_FLAG_NEXT_CTRL;
-		ret = doioctl(node, VIDIOC_QUERYCTRL, &qctrl);
+		ret = doioctl(node, VIDIOC_QUERY_EXT_CTRL, &qctrl);
 		if (ret == ENOTTY)
 			return ret;
 		if (ret && ret != EINVAL)
-			return fail("invalid queryctrl return code (%d)\n", ret);
+			return fail("invalid query_ext_ctrl return code (%d)\n", ret);
 		if (ret)
 			break;
 		if (checkQCtrl(node, qctrl))
@@ -204,12 +207,12 @@ int testQueryControls(struct node *node)
 		id = qctrl.id;
 		if (id >= V4L2_CID_PRIVATE_BASE)
 			return fail("no V4L2_CID_PRIVATE_BASE allowed\n");
-		if (V4L2_CTRL_ID2CLASS(id) != ctrl_class) {
-			if (ctrl_class && !found_ctrl_class)
-				return fail("missing control class for class %08x\n", ctrl_class);
-			if (ctrl_class && !class_count)
-				return fail("no controls in class %08x\n", ctrl_class);
-			ctrl_class = V4L2_CTRL_ID2CLASS(id);
+		if (V4L2_CTRL_ID2WHICH(id) != which) {
+			if (which && !found_ctrl_class)
+				result = fail("missing control class for class %08x\n", which);
+			if (which && !class_count)
+				return fail("no controls in class %08x\n", which);
+			which = V4L2_CTRL_ID2WHICH(id);
 			found_ctrl_class = false;
 			class_count = 0;
 		}
@@ -219,7 +222,7 @@ int testQueryControls(struct node *node)
 			class_count++;
 		}
 
-		if (ctrl_class == V4L2_CTRL_CLASS_USER &&
+		if (which == V4L2_CTRL_CLASS_USER &&
 		    qctrl.type != V4L2_CTRL_TYPE_INTEGER64 &&
 		    qctrl.type != V4L2_CTRL_TYPE_STRING &&
 		    qctrl.type != V4L2_CTRL_TYPE_CTRL_CLASS) {
@@ -232,19 +235,19 @@ int testQueryControls(struct node *node)
 			node->priv_controls++;
 		else
 			node->std_controls++;
-		node->controls.push_back(qctrl);
+		node->controls[qctrl.id] = qctrl;
 	}
-	if (ctrl_class && !found_ctrl_class)
-		return fail("missing control class for class %08x\n", ctrl_class);
-	if (ctrl_class && !class_count)
-		return fail("no controls in class %08x\n", ctrl_class);
+	if (which && !found_ctrl_class)
+		result = fail("missing control class for class %08x\n", which);
+	if (which && !class_count)
+		return fail("no controls in class %08x\n", which);
 
 	for (id = V4L2_CID_BASE; id < V4L2_CID_LASTP1; id++) {
 		memset(&qctrl, 0xff, sizeof(qctrl));
 		qctrl.id = id;
-		ret = doioctl(node, VIDIOC_QUERYCTRL, &qctrl);
+		ret = doioctl(node, VIDIOC_QUERY_EXT_CTRL, &qctrl);
 		if (ret && ret != EINVAL)
-			return fail("invalid queryctrl return code (%d)\n", ret);
+			return fail("invalid query_ext_ctrl return code (%d)\n", ret);
 		if (ret)
 			continue;
 		if (qctrl.id != id)
@@ -258,9 +261,9 @@ int testQueryControls(struct node *node)
 	for (id = V4L2_CID_PRIVATE_BASE; ; id++) {
 		memset(&qctrl, 0xff, sizeof(qctrl));
 		qctrl.id = id;
-		ret = doioctl(node, VIDIOC_QUERYCTRL, &qctrl);
+		ret = doioctl(node, VIDIOC_QUERY_EXT_CTRL, &qctrl);
 		if (ret && ret != EINVAL)
-			return fail("invalid queryctrl return code (%d)\n", ret);
+			return fail("invalid query_ext_ctrl return code (%d)\n", ret);
 		if (ret)
 			break;
 		if (qctrl.id != id)
@@ -279,10 +282,61 @@ int testQueryControls(struct node *node)
 	if (priv_user_controls != priv_user_controls_check)
 		return fail("expected %d private controls, got %d\n",
 			priv_user_controls_check, priv_user_controls);
+	return result;
+}
+
+int testQueryControls(struct node *node)
+{
+	struct v4l2_queryctrl qctrl;
+	unsigned controls = 0;
+	__u32 id = 0;
+	int ret;
+
+	for (;;) {
+		memset(&qctrl, 0xff, sizeof(qctrl));
+		qctrl.id = id | V4L2_CTRL_FLAG_NEXT_CTRL;
+		ret = doioctl(node, VIDIOC_QUERYCTRL, &qctrl);
+		if (ret == ENOTTY)
+			return ret;
+		if (ret && ret != EINVAL)
+			return fail("invalid queryctrl return code (%d)\n", ret);
+		if (ret)
+			break;
+		id = qctrl.id;
+		fail_on_test(node->controls.find(qctrl.id) == node->controls.end());
+		controls++;
+	}
+	fail_on_test(node->controls.size() != controls);
+
+	for (id = V4L2_CID_BASE; id < V4L2_CID_LASTP1; id++) {
+		memset(&qctrl, 0xff, sizeof(qctrl));
+		qctrl.id = id;
+		ret = doioctl(node, VIDIOC_QUERYCTRL, &qctrl);
+		if (ret && ret != EINVAL)
+			return fail("invalid queryctrl return code (%d)\n", ret);
+		if (ret)
+			continue;
+		if (qctrl.id != id)
+			return fail("qctrl.id (%08x) != id (%08x)\n",
+					qctrl.id, id);
+	}
+
+	for (id = V4L2_CID_PRIVATE_BASE; ; id++) {
+		memset(&qctrl, 0xff, sizeof(qctrl));
+		qctrl.id = id;
+		ret = doioctl(node, VIDIOC_QUERYCTRL, &qctrl);
+		if (ret && ret != EINVAL)
+			return fail("invalid queryctrl return code (%d)\n", ret);
+		if (ret)
+			break;
+		if (qctrl.id != id)
+			return fail("qctrl.id (%08x) != id (%08x)\n",
+					qctrl.id, id);
+	}
 	return 0;
 }
 
-static int checkSimpleCtrl(struct v4l2_control &ctrl, struct test_queryctrl &qctrl)
+static int checkSimpleCtrl(struct v4l2_control &ctrl, struct test_query_ext_ctrl &qctrl)
 {
 	if (ctrl.id != qctrl.id)
 		return fail("control id mismatch\n");
@@ -302,7 +356,7 @@ static int checkSimpleCtrl(struct v4l2_control &ctrl, struct test_queryctrl &qct
 		}
 		break;
 	case V4L2_CTRL_TYPE_BITMASK:
-		if (ctrl.value & ~qctrl.maximum)
+		if ((__u32)ctrl.value & ~qctrl.maximum)
 			return fail("returned control value out of range\n");
 		break;
 	case V4L2_CTRL_TYPE_BUTTON:
@@ -315,50 +369,52 @@ static int checkSimpleCtrl(struct v4l2_control &ctrl, struct test_queryctrl &qct
 
 int testSimpleControls(struct node *node)
 {
-	qctrl_list::iterator iter;
+	qctrl_map::iterator iter;
 	struct v4l2_control ctrl;
 	int ret;
 	int i;
 
 	for (iter = node->controls.begin(); iter != node->controls.end(); ++iter) {
-		info("checking control '%s' (0x%08x)\n", iter->name, iter->id);
-		ctrl.id = iter->id;
-		if (iter->type == V4L2_CTRL_TYPE_INTEGER64 ||
-		    iter->type == V4L2_CTRL_TYPE_STRING ||
-		    iter->type == V4L2_CTRL_TYPE_CTRL_CLASS) {
+		test_query_ext_ctrl &qctrl = iter->second;
+
+		info("checking control '%s' (0x%08x)\n", qctrl.name, qctrl.id);
+		ctrl.id = qctrl.id;
+		if (qctrl.type == V4L2_CTRL_TYPE_INTEGER64 ||
+		    qctrl.type == V4L2_CTRL_TYPE_STRING ||
+		    qctrl.type == V4L2_CTRL_TYPE_CTRL_CLASS) {
 			ret = doioctl(node, VIDIOC_G_CTRL, &ctrl);
 			if (ret != EINVAL &&
-			    !((iter->flags & V4L2_CTRL_FLAG_WRITE_ONLY) && ret == EACCES))
+			    !((qctrl.flags & V4L2_CTRL_FLAG_WRITE_ONLY) && ret == EACCES))
 				return fail("g_ctrl allowed for unsupported type\n");
-			ctrl.id = iter->id;
+			ctrl.id = qctrl.id;
 			ctrl.value = 0;
 			// This call will crash on kernels <= 2.6.37 for control classes due to
 			// a bug in v4l2-ctrls.c. So skip this on those kernels.
-			if (kernel_version < 38 && iter->type == V4L2_CTRL_TYPE_CTRL_CLASS)
+			if (kernel_version < 38 && qctrl.type == V4L2_CTRL_TYPE_CTRL_CLASS)
 				ret = EACCES;
 			else
 				ret = doioctl(node, VIDIOC_S_CTRL, &ctrl);
 			if (ret != EINVAL &&
-			    !((iter->flags & V4L2_CTRL_FLAG_READ_ONLY) && ret == EACCES))
+			    !((qctrl.flags & V4L2_CTRL_FLAG_READ_ONLY) && ret == EACCES))
 				return fail("s_ctrl allowed for unsupported type\n");
 			continue;
 		}
 
 		// Get the current value
 		ret = doioctl(node, VIDIOC_G_CTRL, &ctrl);
-		if ((iter->flags & V4L2_CTRL_FLAG_WRITE_ONLY)) {
+		if ((qctrl.flags & V4L2_CTRL_FLAG_WRITE_ONLY)) {
 			if (ret != EACCES)
 				return fail("g_ctrl did not check the write-only flag\n");
-			ctrl.id = iter->id;
-			ctrl.value = iter->default_value;
+			ctrl.id = qctrl.id;
+			ctrl.value = qctrl.default_value;
 		} else if (ret)
 			return fail("g_ctrl returned an error (%d)\n", ret);
-		else if (checkSimpleCtrl(ctrl, *iter))
-			return fail("invalid control %08x\n", iter->id);
+		else if (checkSimpleCtrl(ctrl, qctrl))
+			return fail("invalid control %08x\n", qctrl.id);
 		
 		// Try to set the current value (or the default value for write only controls)
 		ret = doioctl(node, VIDIOC_S_CTRL, &ctrl);
-		if (iter->flags & V4L2_CTRL_FLAG_READ_ONLY) {
+		if (qctrl.flags & V4L2_CTRL_FLAG_READ_ONLY) {
 			if (ret != EACCES)
 				return fail("s_ctrl did not check the read-only flag\n");
 		} else if (ret == EIO) {
@@ -369,47 +425,53 @@ int testSimpleControls(struct node *node)
 		}
 		if (ret)
 			continue;
-		if (checkSimpleCtrl(ctrl, *iter))
-			return fail("s_ctrl returned invalid control contents (%08x)\n", iter->id);
+		if (checkSimpleCtrl(ctrl, qctrl))
+			return fail("s_ctrl returned invalid control contents (%08x)\n", qctrl.id);
 
 		// Try to set value 'minimum - 1'
-		if ((unsigned)iter->minimum != 0x80000000) {
-			ctrl.id = iter->id;
-			ctrl.value = iter->minimum - 1;
+		if (qctrl.minimum > -0x80000000LL && qctrl.minimum <= 0x7fffffffLL) {
+			ctrl.id = qctrl.id;
+			ctrl.value = qctrl.minimum - 1;
 			ret = doioctl(node, VIDIOC_S_CTRL, &ctrl);
-			if (ret && ret != ERANGE)
+			if (ret && ret != EIO && ret != ERANGE)
 				return fail("invalid minimum range check\n");
-			if (!ret && checkSimpleCtrl(ctrl, *iter))
-				return fail("invalid control %08x\n", iter->id);
+			if (!ret && checkSimpleCtrl(ctrl, qctrl))
+				return fail("invalid control %08x\n", qctrl.id);
 		}
 		// Try to set value 'maximum + 1'
-		if ((unsigned)iter->maximum != 0x7fffffff) {
-			ctrl.id = iter->id;
-			ctrl.value = iter->maximum + 1;
+		if (qctrl.maximum >= -0x80000000LL && qctrl.maximum < 0x7fffffffLL) {
+			ctrl.id = qctrl.id;
+			ctrl.value = qctrl.maximum + 1;
 			ret = doioctl(node, VIDIOC_S_CTRL, &ctrl);
-			if (ret && ret != ERANGE)
+			if (ret && ret != EIO && ret != ERANGE)
 				return fail("invalid maximum range check\n");
-			if (!ret && checkSimpleCtrl(ctrl, *iter))
-				return fail("invalid control %08x\n", iter->id);
+			if (!ret && checkSimpleCtrl(ctrl, qctrl))
+				return fail("invalid control %08x\n", qctrl.id);
 		}
 		// Try to set non-step value
-		if (iter->step > 1 && iter->maximum > iter->minimum) {
-			ctrl.id = iter->id;
-			ctrl.value = iter->minimum + 1;
+		if (qctrl.step > 1 && qctrl.maximum > qctrl.minimum) {
+			ctrl.id = qctrl.id;
+			ctrl.value = qctrl.minimum + 1;
 			ret = doioctl(node, VIDIOC_S_CTRL, &ctrl);
 			if (ret == ERANGE)
 				warn("%s: returns ERANGE for in-range, but non-step-multiple value\n",
-						iter->name);
-			else if (ret)
+						qctrl.name);
+			else if (ret && ret != EIO)
 				return fail("returns error for in-range, but non-step-multiple value\n");
 		}
 
-		if (iter->type == V4L2_CTRL_TYPE_MENU || iter->type == V4L2_CTRL_TYPE_INTEGER_MENU) {
-			// check menu items
-			for (i = iter->minimum; i <= iter->maximum; i++) {
-				unsigned valid = iter->menu_mask & (1 << i);
+		if (qctrl.type == V4L2_CTRL_TYPE_MENU || qctrl.type == V4L2_CTRL_TYPE_INTEGER_MENU) {
+			int max = qctrl.maximum;
 
-				ctrl.id = iter->id; 
+			/* Currently menu_mask is a 64-bit value, so we can't reliably
+			 * test for menu item > 63. */
+			if (max > 63)
+				max = 63;
+			// check menu items
+			for (i = qctrl.minimum; i <= max; i++) {
+				bool valid = qctrl.menu_mask & (1ULL << i);
+
+				ctrl.id = qctrl.id; 
 				ctrl.value = i;
 				ret = doioctl(node, VIDIOC_S_CTRL, &ctrl);
 				if (valid && ret)
@@ -421,18 +483,18 @@ int testSimpleControls(struct node *node)
 			}
 		} else {
 			// at least min, max and default values should work
-			ctrl.id = iter->id; 
-			ctrl.value = iter->minimum;
+			ctrl.id = qctrl.id; 
+			ctrl.value = qctrl.minimum;
 			ret = doioctl(node, VIDIOC_S_CTRL, &ctrl);
-			if (ret)
+			if (ret && ret != EIO)
 				return fail("could not set minimum value\n");
-			ctrl.value = iter->maximum;
+			ctrl.value = qctrl.maximum;
 			ret = doioctl(node, VIDIOC_S_CTRL, &ctrl);
-			if (ret)
+			if (ret && ret != EIO)
 				return fail("could not set maximum value\n");
-			ctrl.value = iter->default_value;
+			ctrl.value = qctrl.default_value;
 			ret = doioctl(node, VIDIOC_S_CTRL, &ctrl);
-			if (ret)
+			if (ret && ret != EIO)
 				return fail("could not set default value\n");
 		}
 	}
@@ -450,7 +512,7 @@ int testSimpleControls(struct node *node)
 	return 0;
 }
 
-static int checkExtendedCtrl(struct v4l2_ext_control &ctrl, struct test_queryctrl &qctrl)
+static int checkExtendedCtrl(struct v4l2_ext_control &ctrl, struct test_query_ext_ctrl &qctrl)
 {
 	int len;
 
@@ -458,6 +520,7 @@ static int checkExtendedCtrl(struct v4l2_ext_control &ctrl, struct test_queryctr
 		return fail("control id mismatch\n");
 	switch (qctrl.type) {
 	case V4L2_CTRL_TYPE_INTEGER:
+	case V4L2_CTRL_TYPE_INTEGER64:
 	case V4L2_CTRL_TYPE_BOOLEAN:
 	case V4L2_CTRL_TYPE_MENU:
 	case V4L2_CTRL_TYPE_INTEGER_MENU:
@@ -472,7 +535,7 @@ static int checkExtendedCtrl(struct v4l2_ext_control &ctrl, struct test_queryctr
 		}
 		break;
 	case V4L2_CTRL_TYPE_BITMASK:
-		if (ctrl.value & ~qctrl.maximum)
+		if ((__u32)ctrl.value & ~qctrl.maximum)
 			return fail("returned control value out of range\n");
 		break;
 	case V4L2_CTRL_TYPE_BUTTON:
@@ -494,12 +557,12 @@ static int checkExtendedCtrl(struct v4l2_ext_control &ctrl, struct test_queryctr
 
 int testExtendedControls(struct node *node)
 {
-	qctrl_list::iterator iter;
+	qctrl_map::iterator iter;
 	struct v4l2_ext_controls ctrls;
 	std::vector<struct v4l2_ext_control> total_vec;
 	std::vector<struct v4l2_ext_control> class_vec;
 	struct v4l2_ext_control ctrl;
-	__u32 ctrl_class = 0;
+	__u32 which = 0;
 	bool multiple_classes = false;
 	int ret;
 
@@ -511,8 +574,8 @@ int testExtendedControls(struct node *node)
 		return fail("g_ext_ctrls does not support count == 0\n");
 	if (node->controls.empty())
 		return fail("g_ext_ctrls worked even when no controls are present\n");
-	if (ctrls.ctrl_class)
-		return fail("field ctrl_class changed\n");
+	if (ctrls.which)
+		return fail("field which changed\n");
 	if (ctrls.count)
 		return fail("field count changed\n");
 	if (check_0(ctrls.reserved, sizeof(ctrls.reserved)))
@@ -526,41 +589,44 @@ int testExtendedControls(struct node *node)
 		return fail("try_ext_ctrls does not support count == 0\n");
 	if (node->controls.empty())
 		return fail("try_ext_ctrls worked even when no controls are present\n");
-	if (ctrls.ctrl_class)
-		return fail("field ctrl_class changed\n");
+	if (ctrls.which)
+		return fail("field which changed\n");
 	if (ctrls.count)
 		return fail("field count changed\n");
 	if (check_0(ctrls.reserved, sizeof(ctrls.reserved)))
 		return fail("reserved not zeroed\n");
 
 	for (iter = node->controls.begin(); iter != node->controls.end(); ++iter) {
-		info("checking extended control '%s' (0x%08x)\n", iter->name, iter->id);
-		ctrl.id = iter->id;
+		test_query_ext_ctrl &qctrl = iter->second;
+
+		info("checking extended control '%s' (0x%08x)\n", qctrl.name, qctrl.id);
+		ctrl.id = qctrl.id;
 		ctrl.size = 0;
+		ctrl.ptr = NULL;
 		ctrl.reserved2[0] = 0;
 		ctrls.count = 1;
 
 		// Either should work, so try both semi-randomly
-		ctrls.ctrl_class = (ctrl.id & 1) ? 0 : V4L2_CTRL_ID2CLASS(ctrl.id);
+		ctrls.which = (ctrl.id & 1) ? 0 : V4L2_CTRL_ID2WHICH(ctrl.id);
 		ctrls.controls = &ctrl;
 
 		// Get the current value
 		ret = doioctl(node, VIDIOC_G_EXT_CTRLS, &ctrls);
-		if ((iter->flags & V4L2_CTRL_FLAG_WRITE_ONLY)) {
+		if ((qctrl.flags & V4L2_CTRL_FLAG_WRITE_ONLY)) {
 			if (ret != EACCES)
 				return fail("g_ext_ctrls did not check the write-only flag\n");
 			if (ctrls.error_idx != ctrls.count)
 				return fail("invalid error index write only control\n");
-			ctrl.id = iter->id;
-			ctrl.value = iter->default_value;
+			ctrl.id = qctrl.id;
+			ctrl.value = qctrl.default_value;
 		} else {
-			if (ret != ENOSPC && iter->type == V4L2_CTRL_TYPE_STRING)
+			if (ret != ENOSPC && qctrl.type == V4L2_CTRL_TYPE_STRING)
 				return fail("did not check against size\n");
-			if (ret == ENOSPC && iter->type == V4L2_CTRL_TYPE_STRING) {
+			if (ret == ENOSPC && qctrl.type == V4L2_CTRL_TYPE_STRING) {
 				if (ctrls.error_idx != 0)
 					return fail("invalid error index string control\n");
-				ctrl.string = new char[iter->maximum + 1];
-				ctrl.size = iter->maximum + 1;
+				ctrl.string = new char[qctrl.maximum + 1];
+				ctrl.size = qctrl.maximum + 1;
 				ret = doioctl(node, VIDIOC_G_EXT_CTRLS, &ctrls);
 			}
 			if (ret == EIO) {
@@ -569,13 +635,13 @@ int testExtendedControls(struct node *node)
 			}
 			if (ret)
 				return fail("g_ext_ctrls returned an error (%d)\n", ret);
-			if (checkExtendedCtrl(ctrl, *iter))
-				return fail("invalid control %08x\n", iter->id);
+			if (checkExtendedCtrl(ctrl, qctrl))
+				return fail("invalid control %08x\n", qctrl.id);
 		}
 		
 		// Try the current value (or the default value for write only controls)
 		ret = doioctl(node, VIDIOC_TRY_EXT_CTRLS, &ctrls);
-		if (iter->flags & V4L2_CTRL_FLAG_READ_ONLY) {
+		if (qctrl.flags & V4L2_CTRL_FLAG_READ_ONLY) {
 			if (ret != EACCES)
 				return fail("try_ext_ctrls did not check the read-only flag\n");
 			if (ctrls.error_idx != 0)
@@ -586,7 +652,7 @@ int testExtendedControls(struct node *node)
 		
 		// Try to set the current value (or the default value for write only controls)
 		ret = doioctl(node, VIDIOC_S_EXT_CTRLS, &ctrls);
-		if (iter->flags & V4L2_CTRL_FLAG_READ_ONLY) {
+		if (qctrl.flags & V4L2_CTRL_FLAG_READ_ONLY) {
 			if (ret != EACCES)
 				return fail("s_ext_ctrls did not check the read-only flag\n");
 			if (ctrls.error_idx != ctrls.count)
@@ -599,22 +665,23 @@ int testExtendedControls(struct node *node)
 			if (ret)
 				return fail("s_ext_ctrls returned an error (%d)\n", ret);
 		
-			if (checkExtendedCtrl(ctrl, *iter))
-				return fail("s_ext_ctrls returned invalid control contents (%08x)\n", iter->id);
+			if (checkExtendedCtrl(ctrl, qctrl))
+				return fail("s_ext_ctrls returned invalid control contents (%08x)\n", qctrl.id);
 		}
-		if (iter->type == V4L2_CTRL_TYPE_STRING)
+		if (qctrl.type == V4L2_CTRL_TYPE_STRING)
 			delete [] ctrl.string;
 		ctrl.string = NULL;
 	}
 
-	ctrls.ctrl_class = 0;
+	ctrls.which = 0;
 	ctrl.id = 0;
 	ctrl.size = 0;
 	ret = doioctl(node, VIDIOC_G_EXT_CTRLS, &ctrls);
 	if (ret != EINVAL)
 		return fail("g_ext_ctrls accepted invalid control ID\n");
+	fail_on_test(ctrls.error_idx > ctrls.count);
 	if (ctrls.error_idx != ctrls.count)
-		return fail("g_ext_ctrls(0) invalid error_idx %u\n", ctrls.error_idx);
+		warn("g_ext_ctrls(0) invalid error_idx %u\n", ctrls.error_idx);
 	ctrl.id = 0;
 	ctrl.size = 0;
 	ctrl.value = 0;
@@ -630,20 +697,21 @@ int testExtendedControls(struct node *node)
 		return fail("s_ext_ctrls(0) invalid error_idx %u\n", ctrls.error_idx);
 
 	for (iter = node->controls.begin(); iter != node->controls.end(); ++iter) {
+		test_query_ext_ctrl &qctrl = iter->second;
 		struct v4l2_ext_control ctrl;
 
-		if (iter->flags & (V4L2_CTRL_FLAG_READ_ONLY | V4L2_CTRL_FLAG_WRITE_ONLY))
+		if (qctrl.flags & (V4L2_CTRL_FLAG_READ_ONLY | V4L2_CTRL_FLAG_WRITE_ONLY))
 			continue;
-		ctrl.id = iter->id;
+		ctrl.id = qctrl.id;
 		ctrl.size = 0;
-		if (iter->type == V4L2_CTRL_TYPE_STRING) {
-			ctrl.size = iter->maximum + 1;
+		if (qctrl.type == V4L2_CTRL_TYPE_STRING) {
+			ctrl.size = qctrl.maximum + 1;
 			ctrl.string = new char[ctrl.size];
 		}
 		ctrl.reserved2[0] = 0;
-		if (!ctrl_class)
-			ctrl_class = V4L2_CTRL_ID2CLASS(ctrl.id);
-		else if (ctrl_class != V4L2_CTRL_ID2CLASS(ctrl.id))
+		if (!which)
+			which = V4L2_CTRL_ID2WHICH(ctrl.id);
+		else if (which != V4L2_CTRL_ID2WHICH(ctrl.id))
 			multiple_classes = true;
 		total_vec.push_back(ctrl);
 	}
@@ -664,14 +732,14 @@ int testExtendedControls(struct node *node)
 	if (ret)
 		return fail("could not set all controls\n");
 
-	ctrls.ctrl_class = ctrl_class;
+	ctrls.which = which;
 	ret = doioctl(node, VIDIOC_G_EXT_CTRLS, &ctrls);
 	if (ret && !multiple_classes)
 		return fail("could not get all controls of a specific class\n");
 	if (ret != EINVAL && multiple_classes)
 		return fail("should get EINVAL when getting mixed-class controls\n");
 	if (multiple_classes && ctrls.error_idx != ctrls.count)
-		return fail("error_idx should be equal to count\n");
+		warn("error_idx should be equal to count\n");
 	ret = doioctl(node, VIDIOC_TRY_EXT_CTRLS, &ctrls);
 	if (ret && !multiple_classes)
 		return fail("could not try all controls of a specific class\n");
@@ -689,48 +757,49 @@ int testExtendedControls(struct node *node)
 	if (ret != EINVAL && multiple_classes)
 		return fail("should get EINVAL when setting mixed-class controls\n");
 	if (multiple_classes && ctrls.error_idx != ctrls.count)
-		return fail("error_idx should be equal to count\n");
+		warn("error_idx should be equal to count\n");
 	return 0;
 }
 
 int testControlEvents(struct node *node)
 {
-	qctrl_list::iterator iter;
+	qctrl_map::iterator iter;
 
 	for (iter = node->controls.begin(); iter != node->controls.end(); ++iter) {
+		test_query_ext_ctrl &qctrl = iter->second;
 		struct v4l2_event_subscription sub = { 0 };
 		struct v4l2_event ev;
 		struct timeval timeout = { 0, 100 };
 		fd_set set;
 		int ret;
 
-		info("checking control event '%s' (0x%08x)\n", iter->name, iter->id);
+		info("checking control event '%s' (0x%08x)\n", qctrl.name, qctrl.id);
 		sub.type = V4L2_EVENT_CTRL;
-		sub.id = iter->id;
+		sub.id = qctrl.id;
 		sub.flags = V4L2_EVENT_SUB_FL_SEND_INITIAL;
 		ret = doioctl(node, VIDIOC_SUBSCRIBE_EVENT, &sub);
 		if (ret)
-			return fail("subscribe event for control '%s' failed\n", iter->name);
-		//if (iter->type == V4L2_CTRL_TYPE_CTRL_CLASS)
+			return fail("subscribe event for control '%s' failed\n", qctrl.name);
+		//if (qctrl.type == V4L2_CTRL_TYPE_CTRL_CLASS)
 		FD_ZERO(&set);
-		FD_SET(node->fd, &set);
-		ret = select(node->fd + 1, NULL, NULL, &set, &timeout);
+		FD_SET(node->g_fd(), &set);
+		ret = select(node->g_fd() + 1, NULL, NULL, &set, &timeout);
 		if (ret == 0) {
-			if (iter->type != V4L2_CTRL_TYPE_CTRL_CLASS)
-				return fail("failed to find event for control '%s'\n", iter->name);
-		} else if (iter->type == V4L2_CTRL_TYPE_CTRL_CLASS) {
-			return fail("found event for control class '%s'\n", iter->name);
+			if (qctrl.type != V4L2_CTRL_TYPE_CTRL_CLASS)
+				return fail("failed to find event for control '%s'\n", qctrl.name);
+		} else if (qctrl.type == V4L2_CTRL_TYPE_CTRL_CLASS) {
+			return fail("found event for control class '%s'\n", qctrl.name);
 		}
 		if (ret) {
 			ret = doioctl(node, VIDIOC_DQEVENT, &ev);
 			if (ret)
-				return fail("couldn't get event for control '%s'\n", iter->name);
-			if (ev.type != V4L2_EVENT_CTRL || ev.id != iter->id)
+				return fail("couldn't get event for control '%s'\n", qctrl.name);
+			if (ev.type != V4L2_EVENT_CTRL || ev.id != qctrl.id)
 				return fail("dequeued wrong event\n");
 		}
 		ret = doioctl(node, VIDIOC_UNSUBSCRIBE_EVENT, &sub);
 		if (ret)
-			return fail("unsubscribe event for control '%s' failed\n", iter->name);
+			return fail("unsubscribe event for control '%s' failed\n", qctrl.name);
 	}
 	if (node->controls.empty())
 		return ENOTTY;
