@@ -30,7 +30,12 @@
 #include <sys/ioctl.h>
 #include <sys/time.h>
 #include <math.h>
+
+#ifdef ANDROID
+#include <android-config.h>
+#else
 #include <config.h>
+#endif
 
 #ifdef HAVE_SYS_KLOG_H
 #include <sys/klog.h>
@@ -191,8 +196,16 @@ static std::string cap2s(unsigned cap)
 
 	if (cap & V4L2_CAP_VIDEO_CAPTURE)
 		s += "\t\tVideo Capture\n";
+	if (cap & V4L2_CAP_VIDEO_CAPTURE_MPLANE)
+		s += "\t\tVideo Capture Multiplanar\n";
 	if (cap & V4L2_CAP_VIDEO_OUTPUT)
 		s += "\t\tVideo Output\n";
+	if (cap & V4L2_CAP_VIDEO_OUTPUT_MPLANE)
+		s += "\t\tVideo Output Multiplanar\n";
+	if (cap & V4L2_CAP_VIDEO_M2M)
+		s += "\t\tVideo Memory-to-Memory\n";
+	if (cap & V4L2_CAP_VIDEO_M2M_MPLANE)
+		s += "\t\tVideo Memory-to-Memory Multiplanar\n";
 	if (cap & V4L2_CAP_VIDEO_OVERLAY)
 		s += "\t\tVideo Overlay\n";
 	if (cap & V4L2_CAP_VIDEO_OUTPUT_OVERLAY)
@@ -207,8 +220,16 @@ static std::string cap2s(unsigned cap)
 		s += "\t\tSliced VBI Output\n";
 	if (cap & V4L2_CAP_RDS_CAPTURE)
 		s += "\t\tRDS Capture\n";
+	if (cap & V4L2_CAP_RDS_OUTPUT)
+		s += "\t\tRDS Output\n";
+	if (cap & V4L2_CAP_SDR_CAPTURE)
+		s += "\t\tSDR Capture\n";
 	if (cap & V4L2_CAP_TUNER)
 		s += "\t\tTuner\n";
+	if (cap & V4L2_CAP_HW_FREQ_SEEK)
+		s += "\t\tHW Frequency Seek\n";
+	if (cap & V4L2_CAP_MODULATOR)
+		s += "\t\tModulator\n";
 	if (cap & V4L2_CAP_AUDIO)
 		s += "\t\tAudio\n";
 	if (cap & V4L2_CAP_RADIO)
@@ -219,6 +240,10 @@ static std::string cap2s(unsigned cap)
 		s += "\t\tAsync I/O\n";
 	if (cap & V4L2_CAP_STREAMING)
 		s += "\t\tStreaming\n";
+	if (cap & V4L2_CAP_EXT_PIX_FORMAT)
+		s += "\t\tExtended Pix Format\n";
+	if (cap & V4L2_CAP_DEVICE_CAPS)
+		s += "\t\tDevice Capabilities\n";
 	return s;
 }
 
@@ -375,7 +400,7 @@ int main(int argc, char **argv)
 	struct v4l2_dbg_register get_reg;
 	struct v4l2_dbg_chip_info chip_info;
 	const struct board_list *curr_bd = NULL;
-	char short_options[26 * 2 * 2 + 1];
+	char short_options[26 * 2 * 3 + 1];
 	int idx = 0;
 	std::string reg_min_arg, reg_max_arg;
 	std::string reg_set_arg;
@@ -397,8 +422,12 @@ int main(int argc, char **argv)
 		if (!isalpha(long_options[i].val))
 			continue;
 		short_options[idx++] = long_options[i].val;
-		if (long_options[i].has_arg == required_argument)
+		if (long_options[i].has_arg == required_argument) {
 			short_options[idx++] = ':';
+		} else if (long_options[i].has_arg == optional_argument) {
+			short_options[idx++] = ':';
+			short_options[idx++] = ':';
+		}
 	}
 	while (1) {
 		int option_index = 0;
@@ -545,6 +574,12 @@ int main(int argc, char **argv)
 			usage();
 		set_reg.reg = parse_reg(curr_bd, reg_set_arg);
 		while (optind < argc) {
+			unsigned size = 0;
+
+			if (doioctl(fd, VIDIOC_DBG_G_REGISTER, &set_reg,
+				    "VIDIOC_DBG_G_REGISTER") >= 0)
+				size = set_reg.size;
+
 			set_reg.val = strtoull(argv[optind++], NULL, 0);
 			if (doioctl(fd, VIDIOC_DBG_S_REGISTER, &set_reg,
 						"VIDIOC_DBG_S_REGISTER") >= 0) {
@@ -562,7 +597,7 @@ int main(int argc, char **argv)
 				printf("Failed to set register 0x%08llx value 0x%llx: %s\n",
 					set_reg.reg, set_reg.val, strerror(errno));
 			}
-			set_reg.reg++;
+			set_reg.reg += size ? : (forcedstride ? : 1);
 		}
 	}
 
@@ -718,14 +753,16 @@ list_done:
 			len = klogctl(3, buf, sizeof(buf) - 1);
 #endif
 			if (len >= 0) {
+				bool found_status = false;
 				char *p = buf;
 				char *q;
 
 				buf[len] = 0;
 				while ((q = strstr(p, "START STATUS"))) {
+					found_status = true;
 					p = q + 1;
 				}
-				if (p) {
+				if (found_status) {
 					while (p > buf && *p != '<') p--;
 					q = p;
 					while ((q = strstr(q, "<6>"))) {
